@@ -5,7 +5,7 @@
 // Name:        appconfig.h
 // Purpose:     Application configuration class, deal with configuration
 // Author:      PCMan (HZY)   http://pcman.ptt.cc/
-// E-mail:      hzysoft@sina.com.tw
+// E-mail:      pcman.tw@gmail.com
 // Created:     2004.7.20
 // Copyright:   (C) 2004 PCMan
 // Licence:     GPL : http://www.gnu.org/licenses/gpl.html
@@ -96,7 +96,9 @@ public:
 	bool ShowTrayIcon;
 	bool ShowStatusBar;
 	bool PopupNotifier;
+#ifdef USE_WGET
 	bool UseWgetFiles;
+#endif
 	int PopupTimeout;
 	
 //	Terminal Settings
